# include "dict.h"
#include "reciteword.h"
# include "skin.h"
#include "readword.h"
#include "configfile.h"
#include <gdk/gdkkeysyms.h>


#define XDICT_PATH  "dicts"

extern CReciteWord *g_pReciteWord;
extern ConfigFile *usercfgfile;
extern struct _Skin Skin;

static gint
on_window_delete_event (GtkWidget * window, GdkEvent *event , CDict * parent)
{
	parent->close();
	return true;
}

static gboolean
on_window_key_press(GtkWidget * window, GdkEventKey * event ,CDict * parent)
{
	gboolean return_val=true;
	const char* text;
	switch (event->keyval)
	{
		case GDK_Escape:
			text = gtk_entry_get_text(GTK_ENTRY(parent->word_entry));
			if (text && text[0]!='\0')
			{
				gtk_entry_set_text(GTK_ENTRY(parent->word_entry),"");
			}
			else
			{
				parent->exit_button.do_clicked();
			}
			break;
		default:
			return_val=false;
	}
	return return_val;
}

static void
on_exit_clicked (gpointer data)
{
	CDict * parent = (CDict *)data;
	parent->close ();
}

static void
on_search_clicked (gpointer data)
{
	CDict * parent = (CDict *)data;
	readword(parent->word_text);
}
static void
on_sound_clicked (gpointer data)
{
}
static void
on_card_clicked (gpointer data)
{
}

static void
on_dict_word_entry_activate (GtkWidget * widget, CDict * parent)
{
	on_search_clicked (parent);
}
static void
on_dict_word_entry_changed (GtkWidget * widget, CDict * parent)
{
	parent->word_text =
		gtk_entry_get_text (GTK_ENTRY (widget));
	if (!(parent->word_text))
		return;
	if (parent->word_text[0] == '\0')
		return;
	if (parent->ecdict->working)
	{
		parent->ecdict->query (parent->word_text);
		parent->set_wordlist_label (1);
	}
}

CDict::CDict ()
{
	onlyshow=FALSE;
	if (g_pReciteWord->window)
		skin_load_dict (g_pReciteWord->window);
	ecdict = new mydict (XDICT_PATH);
}

CDict::~CDict ()
{
	delete ecdict;
}

void
CDict::init ()
{
	if (!onlyshow)
		g_pReciteWord->dict_button.set_enable (FALSE);
	word_text = NULL;
}

void
CDict::show ()
{
	init ();
	GtkWidget *fixed;
	//GtkWidget *event_box;
	//GtkWidget *pixmap_widget;
	//gchar str[256];
	
	playsound(SND_WIND);
	showing = TRUE;
	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_size_request (window, Skin.dict.dict.w, Skin.dict.dict.h);
	gtk_window_set_resizable(GTK_WINDOW (window), FALSE);
	gtk_widget_set_app_paintable(window,TRUE);
	gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
	gtk_window_set_title (GTK_WINDOW (window), "Dict");
	gtk_widget_realize (window);
	if (onlyshow)
	{
		skin_load_dict (window);
	}
	else
	{
		gtk_window_set_transient_for (GTK_WINDOW (window),
				      GTK_WINDOW (g_pReciteWord->window));
	}

	g_signal_connect (G_OBJECT (window), "delete_event",
			  G_CALLBACK (on_window_delete_event), this);
	g_signal_connect (G_OBJECT (window), "key_press_event",
			  G_CALLBACK (on_window_key_press), this);


	fixed = gtk_fixed_new ();
	//gtk_fixed_set_has_window(GTK_FIXED(fixed),TRUE);
	gtk_container_add (GTK_CONTAINER (window), fixed);
	gtk_widget_show (fixed);
	SetBackPixmap (fixed, Skin.dict.dict.p);

	exit_button.create (fixed, Skin.dict.exit_button.x,
			    Skin.dict.exit_button.y,
			    Skin.dict.exit_button.p1,
			    Skin.dict.exit_button.p2,
			    Skin.dict.exit_button.p3, on_exit_clicked, this);
	search_button.create (fixed, Skin.dict.search_button.x,
			      Skin.dict.search_button.y,
			      Skin.dict.search_button.p1,
			      Skin.dict.search_button.p2,
			      Skin.dict.search_button.p3, on_search_clicked,
			      this);
	sound_button.create (fixed, Skin.dict.sound_button.x,
			     Skin.dict.sound_button.y,
			     Skin.dict.sound_button.p1,
			     Skin.dict.sound_button.p2,
			     Skin.dict.sound_button.p3, on_sound_clicked,
			     NULL);
	card_button.create (fixed, Skin.dict.card_button.x,
			    Skin.dict.card_button.y, Skin.dict.card_button.p1,
			    Skin.dict.card_button.p2,
			    Skin.dict.card_button.p3, on_card_clicked, NULL);
	search_ckbutton.create (fixed, Skin.dict.search_ckbutton.x,
				Skin.dict.search_ckbutton.y,
				Skin.dict.search_ckbutton.p1,
				Skin.dict.search_ckbutton.p2,
				Skin.dict.search_ckbutton.p3,
				Skin.dict.search_ckbutton.p4, NULL);
	sound_ckbutton.create (fixed, Skin.dict.sound_ckbutton.x,
			       Skin.dict.sound_ckbutton.y,
			       Skin.dict.sound_ckbutton.p1,
			       Skin.dict.sound_ckbutton.p2,
			       Skin.dict.sound_ckbutton.p3,
			       Skin.dict.sound_ckbutton.p4, NULL);
	word_entry = gtk_entry_new ();
	gtk_entry_set_has_frame(GTK_ENTRY(word_entry),false);
	gtk_widget_set_size_request (word_entry, Skin.dict.word_entry.w,
			      Skin.dict.word_entry.h);
	gtk_fixed_put (GTK_FIXED (fixed), word_entry,
		       Skin.dict.word_entry.x, Skin.dict.word_entry.y);
	gtk_entry_set_text (GTK_ENTRY (word_entry), "");

	g_signal_connect (G_OBJECT (word_entry), "activate",
			    G_CALLBACK (on_dict_word_entry_activate),
			    this);
	g_signal_connect (G_OBJECT (word_entry), "changed",
			    G_CALLBACK (on_dict_word_entry_changed),
			    this);
	gtk_widget_show (word_entry);
	gtk_widget_grab_focus (word_entry);
	// word_adjust = gtk_adjustment_new (0, 0,20, 1, 5, 5);
	//gtk_signal_connect (GTK_OBJECT (word_adjust), "value",
	//           GTK_SIGNAL_FUNC (on_type_wordlistadjust_value), this);

	word_vscroll = gtk_vscrollbar_new ((GtkAdjustment *) NULL);
	gtk_widget_set_size_request (word_vscroll, Skin.dict.word_vscroll.w,
			      Skin.dict.word_vscroll.h);
	gtk_fixed_put (GTK_FIXED (fixed), word_vscroll,
		       Skin.dict.word_vscroll.x, Skin.dict.word_vscroll.y);
	gtk_widget_show (word_vscroll);
	/*ying_button.create (NULL,fixed,
	 * Skin.dict.ying_button.x,
	 * Skin.dict.ying_button.y,
	 * Skin.dict.ying_button.p1,
	 * Skin.dict.ying_button.p2,
	 * Skin.dict.ying_button.p3,
	 * Skin.dict.ying_button.p4, NULL);
	 * han_button.create (ying_button.group,fixed,
	 * Skin.dict.han_button.x,
	 * Skin.dict.han_button.y,
	 * Skin.dict.han_button.p1,
	 * Skin.dict.han_button.p2,
	 * Skin.dict.han_button.p3,
	 * Skin.dict.han_button.p4, NULL); */
	wordlist_dict_label[0] = gtk_label_new ("");

	gtk_fixed_put (GTK_FIXED (fixed),
		       wordlist_dict_label[0],
		       Skin.dict.wordlist_dict_label[0].x,
		       Skin.dict.wordlist_dict_label[0].y);

	gtk_widget_show (wordlist_dict_label[0]);
	wordlist_dict_label[1] = gtk_label_new ("");
	gtk_widget_set_size_request (wordlist_dict_label[1], 355, -1);
	gtk_label_set_line_wrap (GTK_LABEL (wordlist_dict_label[1]),
				  TRUE);
	gtk_label_set_justify (GTK_LABEL (wordlist_dict_label[1]),
			       GTK_JUSTIFY_LEFT);
	gtk_fixed_put (GTK_FIXED (fixed), wordlist_dict_label[1],
		       Skin.dict.wordlist_dict_label[1].x,
		       Skin.dict.wordlist_dict_label[1].y);
	gtk_widget_show (wordlist_dict_label[1]);
	if (!ecdict->working)
	{
		gtk_label_set_text(GTK_LABEL(wordlist_dict_label[1]),"Dict init failed,Dict data files not found!");
	}
	gtk_widget_show (window);
}

void
CDict::set_wordlist_label (int index)
{
	const gchar *tmp;
	tmp=ecdict->getNearEnglishItem (index);
	if (tmp)
		gtk_label_set_text (GTK_LABEL (wordlist_dict_label[0]),tmp);
	tmp=ecdict->getNearChineseItem (index);
	if (tmp)
	{
		gchar *tmp1 = g_locale_to_utf8(tmp,-1,NULL,NULL,NULL);
		//sprintf (str, "%s", ecdict->getNearChineseItem (index));
		if (tmp1)
		{
			gtk_label_set_text (GTK_LABEL (wordlist_dict_label[1]), tmp1);
			g_free(tmp1);
		}
	}
}

void
CDict::close ()
{
	playsound(SND_WIND);
	
	search_button.destroy ();
	sound_button.destroy ();
	card_button.destroy ();
	exit_button.destroy ();
	search_ckbutton.destroy ();
	sound_ckbutton.destroy ();
	gtk_widget_destroy (wordlist_dict_label[0]);
	gtk_widget_destroy (wordlist_dict_label[1]);
	gtk_widget_destroy (word_entry);
	gtk_widget_destroy (word_vscroll);
	gtk_widget_destroy (window);
	
	if (onlyshow)
	{
		if (usercfgfile)
		{
			if (usercfgfile->changed)
			{
				rw_cfg_write_file (usercfgfile,
						   usercfgfile->cfgfilename);
			}
			rw_cfg_free (usercfgfile);
			g_free (usercfgfile);
			usercfgfile = NULL;
		}
		gtk_main_quit ();
	}
	else
	{
		showing = FALSE;
		g_pReciteWord->dict_button.set_enable (TRUE);		
	}
}

void
CDict::create()
{
	onlyshow=TRUE;
	show();
	gdk_threads_enter();
	gtk_main ();
	gdk_threads_leave();
}
