#include "shooting.h"
#include "reciteword.h"
#include "skin.h"
#include "sound.h"
#include "readword.h"
#include <cstring>
#include <cstdlib>
#include <ctime>

extern CReciteWord *g_pReciteWord;
extern struct _Skin Skin;


static gint
on_labelitem_expose (GtkWidget * widget, GdkEventExpose * event,
		     tLabelItem * parent)
{
	gdk_draw_drawable (widget->window,
			 widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
			 parent->draw_pixmap,
			 event->area.x, event->area.y,
			 event->area.x, event->area.y,
			 event->area.width, event->area.height);
	return FALSE;
}

static gboolean
on_labelitem_buttonpress (GtkWidget * widget, GdkEventButton * event,
			  tLabelItem * parent)
{
	return TRUE;
}

static gint
on_labelitem_buttonrelease (GtkWidget * widget, GdkEventButton * event,
			    tLabelItem * parent)
{
	if (parent->mousein)
	{
		if (parent->enable)
		{
			(*(parent->runfunc1)) (parent->funcdata);
		}
	}
	return TRUE;
}

static void
on_labelitem_enter_notify (GtkWidget * widget, GdkEvent * event,
			   tLabelItem * parent)
{
	parent->mousein = TRUE;
	if (!parent->enable)
		return;
	(*(parent->runfunc2)) (parent->funcdata);
	parent->active = TRUE;
	parent->draw ();
}

static void
on_labelitem_leave_notify (GtkWidget * widget, GdkEvent * event,
			   tLabelItem * parent)
{
	parent->mousein = FALSE;
	if (!parent->enable)
		return;
	parent->active = FALSE;
	parent->draw ();
}

tLabelItem::tLabelItem ()
{
}

void
tLabelItem::init ()
{
	layout= NULL;
	mousein = FALSE;
	active = FALSE;
	enable = TRUE;
}

void
tLabelItem::create (GtkWidget * pfixed, gint x, gint y, gint w, gint h,
		    GdkPixmap * pix_1, GdkPixmap * pix_2, gchar * str,
		    void (*func1) (gint), void (*func2) (gint), gint data)
{
	init ();

	pix_normal = pix_1;
	pix_active = pix_2;
	width = w;
	height = h;
	runfunc1 = func1;
	runfunc2 = func2;
	funcdata = data;

	draw_area = gtk_drawing_area_new ();
	gtk_widget_set_size_request (draw_area, width, height);

	gtk_widget_set_events (draw_area,
			       GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
			       | GDK_ENTER_NOTIFY_MASK |
			       GDK_LEAVE_NOTIFY_MASK);
	g_signal_connect (G_OBJECT (draw_area), "expose_event",
			  G_CALLBACK (on_labelitem_expose), this);
	g_signal_connect (G_OBJECT (draw_area), "button_press_event",
			  G_CALLBACK (on_labelitem_buttonpress), this);
	g_signal_connect (G_OBJECT (draw_area), "button_release_event",
			  G_CALLBACK (on_labelitem_buttonrelease), this);
	g_signal_connect (G_OBJECT (draw_area), "enter_notify_event",
			  G_CALLBACK (on_labelitem_enter_notify), this);
	g_signal_connect (G_OBJECT (draw_area), "leave_notify_event",
			  G_CALLBACK (on_labelitem_leave_notify), this);

	gtk_fixed_put (GTK_FIXED (pfixed), draw_area, x, y);
	gtk_widget_show (draw_area);
	draw_pixmap =
		gdk_pixmap_new (draw_area->window, width, height,
				gdk_rgb_get_visual ()->depth);
	set_label(str);
}

void
tLabelItem::draw ()
{
	if (active)
		gdk_draw_drawable (draw_pixmap, draw_area->style->white_gc,
				 pix_active, 0, 0, 0, 0, width, height);
	else
		gdk_draw_drawable (draw_pixmap, draw_area->style->white_gc,
				 pix_normal, 0, 0, 0, 0, width, height);
	if (layout)
	{
		gint h,y;
		pango_layout_get_pixel_size(layout,NULL,&h);
		y = (height - h) / 2;
		gdk_draw_layout(draw_pixmap,draw_area->style->white_gc,5,y,layout);
	}
/*	gdk_draw_string (draw_pixmap, gtk_style_get_font (draw_area->style),
			 draw_area->style->white_gc, 5, y, text);
*/
	gtk_widget_queue_draw (draw_area);
}

void
tLabelItem::set_label (gchar * str)
{
	std::strcpy (text, str);
	if (layout)
		pango_layout_set_text(layout,text,-1);
	else
		layout=gtk_widget_create_pango_layout (draw_area, text);
	draw ();
}

void
tLabelItem::set_enable (gint a)
{
	if (enable != a)
	{
		if (a)
		{
		}
		else
		{
			if (active)
			{
				active = FALSE;
				draw ();
			}
		}
		enable = a;
	}
}

void
tLabelItem::destroy ()
{
	if (layout)
		g_object_unref (G_OBJECT (layout));
	gtk_widget_destroy (draw_area);
}

tLabelItem::~tLabelItem ()
{
}



static void
on_shooting_start_clicked ()
{
	if (g_pReciteWord->shooting->started)
	{
		g_pReciteWord->shooting->started = FALSE;
		g_pReciteWord->shooting->wordlist_head[0].set_enable (FALSE);
		g_pReciteWord->shooting->wordlist_head[1].set_enable (FALSE);
		g_pReciteWord->shooting->wordlist_head[2].set_enable (FALSE);
		g_pReciteWord->shooting->wordlist_head[3].set_enable (FALSE);
		g_pReciteWord->shooting->wordlist_item[0].set_enable (FALSE);
		g_pReciteWord->shooting->wordlist_item[1].set_enable (FALSE);
		g_pReciteWord->shooting->wordlist_item[2].set_enable (FALSE);
		g_pReciteWord->shooting->wordlist_item[3].set_enable (FALSE);
		g_pReciteWord->shooting->pause_button.set_enable (FALSE);
		g_pReciteWord->shooting->start_button.reset (Skin.
							     shooting.
							     start_button.
							     p1,
							     Skin.
							     shooting.
							     start_button.
							     p2,
							     Skin.
							     shooting.
							     start_button.
							     p3,
							     (GdkPixmap *)
							     NULL);
	}
	else
	{
		std::srand (std::time (0));
		gint i;
		for (i = 0; i < (g_pReciteWord->now_zu_wordcount); i++)
			g_pReciteWord->shooting->tested_wordlist_item[i] = 0;	// != -1 means didn't tested

		g_pReciteWord->shooting->testing_word_index = 0;

		g_pReciteWord->shooting->wordlist_head[0].set_enable (TRUE);
		g_pReciteWord->shooting->wordlist_head[1].set_enable (TRUE);
		g_pReciteWord->shooting->wordlist_head[2].set_enable (TRUE);
		g_pReciteWord->shooting->wordlist_head[3].set_enable (TRUE);
		g_pReciteWord->shooting->wordlist_item[0].set_enable (TRUE);
		g_pReciteWord->shooting->wordlist_item[1].set_enable (TRUE);
		g_pReciteWord->shooting->wordlist_item[2].set_enable (TRUE);
		g_pReciteWord->shooting->wordlist_item[3].set_enable (TRUE);

		g_pReciteWord->shooting->set_wordlist_label (0);
		g_pReciteWord->shooting->set_wordview_label (0);

		g_pReciteWord->shooting->pause_button.set_enable (TRUE);
		g_pReciteWord->shooting->start_button.reset (Skin.
							     shooting.
							     stop_button.
							     p1,
							     Skin.
							     shooting.
							     stop_button.
							     p2,
							     Skin.
							     shooting.
							     stop_button.
							     p3,
							     Skin.
							     shooting.
							     stop_button.p4);
		g_pReciteWord->shooting->started = TRUE;
	}
}

static void
on_shooting_pause_clicked ()
{
}

static void
on_shooting_return_clicked ()
{
	g_pReciteWord->shooting->close ();
	g_pReciteWord->show ();
}

static void
shooting_test_next_word ()
{
	do
	{
		g_pReciteWord->shooting->testing_word_index++;
		if (g_pReciteWord->shooting->testing_word_index >=
		    g_pReciteWord->now_zu_wordcount)
		{
			int i, b;
			b = 1;
			for (i = 0; i < g_pReciteWord->now_zu_wordcount; i++)
			{
				if (g_pReciteWord->shooting->
				    tested_wordlist_item[i] != -1)
				{
					b = 0;
					break;
				}
			}
			if (b)
			{
				on_shooting_start_clicked ();	//stop

				//g_pReciteWord->shooting->dialog.show();
				return;
			}
			else
			{
				g_pReciteWord->shooting->testing_word_index =
					0;
			}
		}
	}
	while (g_pReciteWord->shooting->
	       tested_wordlist_item[g_pReciteWord->shooting->
				    testing_word_index] == -1);

	g_pReciteWord->shooting->set_wordlist_label (g_pReciteWord->shooting->
						     testing_word_index);
	g_pReciteWord->shooting->set_wordview_label (g_pReciteWord->shooting->
						     testing_word_index);
}

static gint
draw_right_pixmap (gpointer data)
{
	CShooting *parent = g_pReciteWord->shooting;
	gint index = (gint) data;
	GtkWidget *right_pixmap =
		parent->right_pixmap[parent->
				     tested_wordlist_item[parent->
							  testing_word_index]];
	GtkWidget *wordlist_head =
		parent->wordlist_head[parent->
				      tested_wordlist_item[parent->
							   testing_word_index]].
		event_box;

	if (parent->timer_count == 8)
	{
		gtk_widget_hide (right_pixmap);
		shooting_test_next_word ();
		gtk_widget_show (wordlist_head);
		gtk_widget_show (parent->wordlist_head[index].event_box);
		parent->shoot_pixmap_timer = 0;
		return FALSE;
	}
	if (parent->timer_count == -1)
	{
		gtk_widget_hide (wordlist_head);
		parent->timer_count = 0;
	}
	else
		parent->timer_count++;

	if (parent->timer_count % 2)
		gtk_image_set_from_pixmap (GTK_IMAGE (right_pixmap),
				Skin.shooting.right_pixmap[parent->
							   tested_wordlist_item
							   [parent->
							    testing_word_index]].
				p1, (GdkBitmap *) NULL);
	else
		gtk_image_set_from_pixmap (GTK_IMAGE (right_pixmap),
				Skin.shooting.right_pixmap[parent->
							   tested_wordlist_item
							   [parent->
							    testing_word_index]].
				p2, (GdkBitmap *) NULL);
	if (parent->timer_count == 0)
		gtk_widget_show (right_pixmap);
	return TRUE;
}

static gint
draw_shooting (gpointer data)
{
	CShooting *parent = g_pReciteWord->shooting;
	gint index = (gint) data;
	GtkWidget *shoot_pixmap = parent->shoot_pixmap[index];
	GtkWidget *wordlist_head = parent->wordlist_head[index].event_box;

	if (parent->timer_type)
	{
		if (parent->timer_count == 8)
		{
			parent->timer_count++;
			gtk_widget_hide (shoot_pixmap);
			return TRUE;
		}
		if (parent->timer_count == 9)
		{
			shooting_test_next_word ();
			gtk_widget_show (wordlist_head);
			parent->shoot_pixmap_timer = 0;
			return FALSE;
		}
		if (parent->timer_count == -1)
		{
			gtk_widget_hide (wordlist_head);
			parent->timer_count = 0;
		}
		else
			parent->timer_count++;
	}
	else
	{
		if (parent->timer_count == 13)
		{
			gtk_widget_hide (shoot_pixmap);
			parent->timer_count = -1;
			parent->shoot_pixmap_timer =
				gtk_timeout_add (200, draw_right_pixmap,
						 (void *) index);
			return FALSE;
		}
		if (parent->timer_count == -1)
		{
			gtk_widget_hide (wordlist_head);
			parent->timer_count = 9;
		}
		else
			parent->timer_count++;
	}

	gtk_image_set_from_pixmap (GTK_IMAGE (shoot_pixmap),
			Skin.shooting.shoot_pixmap[index].p[parent->
							    timer_count],
			(GdkBitmap *) NULL);
	if ((parent->timer_count == 0) || (parent->timer_count == 9))
		gtk_widget_show (shoot_pixmap);
	return TRUE;
}

static void
draw_shoot_pixmap (gint type, gint index)
{
	CShooting *parent = g_pReciteWord->shooting;
	parent->timer_type = type;
	parent->timer_count = -1;
	parent->shoot_pixmap_timer =
		gtk_timeout_add (40, draw_shooting, (void *) index);
}

static void
on_shooting_wordlistitem_clicked (gint index)
{
	CShooting *parent = g_pReciteWord->shooting;

	if (parent->shoot_pixmap_timer)
		return;

	if (index ==
	    g_pReciteWord->shooting->tested_wordlist_item[g_pReciteWord->
							  shooting->
							  testing_word_index])
	{
		g_pReciteWord->shooting->tested_wordlist_item[g_pReciteWord->
							      shooting->
							      testing_word_index]
			= -1;
		playsound(SND_GUNOK);
		draw_shoot_pixmap (TRUE, index);
	}
	else
	{
		playsound(SND_GUNNO);
		draw_shoot_pixmap (FALSE, index);
	}
}

static void
on_shooting_wordlistitem_active (gint index)
{
	if (g_pReciteWord->shooting->wordlist_item[g_pReciteWord->shooting->active_wordlist_item].active)
	{
		g_pReciteWord->shooting->wordlist_item[g_pReciteWord->shooting->active_wordlist_item].active = false;
		g_pReciteWord->shooting->wordlist_item[g_pReciteWord->shooting->active_wordlist_item].draw ();
	}
	g_pReciteWord->shooting->active_wordlist_item=index;
	switch (index)
	{
	case 0:
		gtk_image_set_from_pixmap (GTK_IMAGE
				(g_pReciteWord->shooting->tank_pixmap),
				Skin.shooting.tank.p1, (GdkBitmap *) NULL);
		break;
	case 1:
		gtk_image_set_from_pixmap (GTK_IMAGE
				(g_pReciteWord->shooting->tank_pixmap),
				Skin.shooting.tank.p2, (GdkBitmap *) NULL);
		break;
	case 2:
		gtk_image_set_from_pixmap (GTK_IMAGE
				(g_pReciteWord->shooting->tank_pixmap),
				Skin.shooting.tank.p3, (GdkBitmap *) NULL);
		break;
	case 3:
		gtk_image_set_from_pixmap (GTK_IMAGE
				(g_pReciteWord->shooting->tank_pixmap),
				Skin.shooting.tank.p4, (GdkBitmap *) NULL);
		break;
	}
}

static void
on_shooting_wordlisthead_button_press (gpointer data)
{
	on_shooting_wordlistitem_clicked ((gint) data);
}


static gboolean
on_shooting_wordlisthead_enter_notify (GtkWidget * widget, GdkEvent * event,
				       gpointer func_data)
{
	if (g_pReciteWord->shooting->shoot_pixmap_timer)	//when mouse button press and release,the enter_notify and leave_notift event will happen too.
		return true;
	gint index = (gint) func_data;
	if (!g_pReciteWord->shooting->wordlist_head[index].enable)
		return true;
	on_shooting_wordlistitem_active (index);
	g_pReciteWord->shooting->wordlist_item[index].active = TRUE;
	g_pReciteWord->shooting->wordlist_item[index].draw ();
	return true;
}

static gboolean
on_shooting_wordlisthead_leave_notify (GtkWidget * widget, GdkEvent * event,
				       gpointer func_data)
{
	if (g_pReciteWord->shooting->shoot_pixmap_timer)
		return true;
	gint index = (gint) func_data;
	if (!g_pReciteWord->shooting->wordlist_head[index].enable)
		return true;
	g_pReciteWord->shooting->wordlist_item[index].active = FALSE;
	g_pReciteWord->shooting->wordlist_item[index].draw ();
	return true;
}

CShooting::CShooting ()
{
	skin_load_shooting (g_pReciteWord->window);
}

void
CShooting::init ()
{
	g_pReciteWord->status = STATUS_SHOOTING;
	status = FALSE;
}

void
CShooting::init_2 ()
{
	shoot_pixmap_timer = 0;
	started = FALSE;
	status = TRUE;
	active_wordlist_item = 0;
	tested_wordlist_item =
		(int *) g_malloc (g_pReciteWord->now_zu_wordcount *
				  (sizeof (int)));
}

static void
shooting_show_2 ()
{
	g_pReciteWord->chooseword->close ();
	g_pReciteWord->shooting->show_2 ();
}

void
CShooting::show ()
{
	init ();
	SetBackPixmap (g_pReciteWord->fixed, Skin.shooting.shooting.p);
	return_button.create (g_pReciteWord->fixed,
			      Skin.shooting.return_button.x,
			      Skin.shooting.return_button.y,
			      Skin.shooting.return_button.p1,
			      Skin.shooting.return_button.p2,
			      Skin.shooting.return_button.p3,
			      (GdkPixmap *) NULL, on_shooting_return_clicked);
	g_pReciteWord->PopupChooseword (shooting_show_2,
					on_shooting_return_clicked);
}

void
CShooting::show_2 ()
{
	init_2 ();

	entocn_robutton.create (NULL, g_pReciteWord->fixed,
				Skin.shooting.entocn_robutton.x,
				Skin.shooting.entocn_robutton.y,
				Skin.shooting.entocn_robutton.p1,
				Skin.shooting.entocn_robutton.p2,
				Skin.shooting.entocn_robutton.p3,
				Skin.shooting.entocn_robutton.p4, NULL);
	cntoen_robutton.create (entocn_robutton.group, g_pReciteWord->fixed,
				Skin.shooting.cntoen_robutton.x,
				Skin.shooting.cntoen_robutton.y,
				Skin.shooting.cntoen_robutton.p1,
				Skin.shooting.cntoen_robutton.p2,
				Skin.shooting.cntoen_robutton.p3,
				Skin.shooting.cntoen_robutton.p4, NULL);

	wordview_label_1 = gtk_label_new ("");
	gtk_widget_set_name (wordview_label_1, "big_white_word_label");
	gtk_fixed_put (GTK_FIXED (g_pReciteWord->fixed), wordview_label_1,
		       Skin.shooting.wordview_label_1.x,
		       Skin.shooting.wordview_label_1.y);
	gtk_widget_show (wordview_label_1);

	read_ckbutton.create (g_pReciteWord->fixed,
			      Skin.shooting.read_ckbutton.x,
			      Skin.shooting.read_ckbutton.y,
			      Skin.shooting.read_ckbutton.p1,
			      Skin.shooting.read_ckbutton.p2,
			      Skin.shooting.read_ckbutton.p3,
			      Skin.shooting.read_ckbutton.p4, NULL);

	wordlist_item[0].create (g_pReciteWord->fixed,
				 Skin.shooting.wordlist_item_1.x,
				 Skin.shooting.wordlist_item_1.y,
				 Skin.shooting.wordlist_item_1.w,
				 Skin.shooting.wordlist_item_1.h,
				 Skin.shooting.wordlist_item_1.p1,
				 Skin.shooting.wordlist_item_1.p2, "",
				 on_shooting_wordlistitem_clicked,
				 on_shooting_wordlistitem_active, 0);
	wordlist_item[1].create (g_pReciteWord->fixed,
				 Skin.shooting.wordlist_item_2.x,
				 Skin.shooting.wordlist_item_2.y,
				 Skin.shooting.wordlist_item_2.w,
				 Skin.shooting.wordlist_item_2.h,
				 Skin.shooting.wordlist_item_2.p1,
				 Skin.shooting.wordlist_item_2.p2, "",
				 on_shooting_wordlistitem_clicked,
				 on_shooting_wordlistitem_active, 1);
	wordlist_item[2].create (g_pReciteWord->fixed,
				 Skin.shooting.wordlist_item_3.x,
				 Skin.shooting.wordlist_item_3.y,
				 Skin.shooting.wordlist_item_3.w,
				 Skin.shooting.wordlist_item_3.h,
				 Skin.shooting.wordlist_item_3.p1,
				 Skin.shooting.wordlist_item_3.p2, "",
				 on_shooting_wordlistitem_clicked,
				 on_shooting_wordlistitem_active, 2);
	wordlist_item[3].create (g_pReciteWord->fixed,
				 Skin.shooting.wordlist_item_4.x,
				 Skin.shooting.wordlist_item_4.y,
				 Skin.shooting.wordlist_item_4.w,
				 Skin.shooting.wordlist_item_4.h,
				 Skin.shooting.wordlist_item_4.p1,
				 Skin.shooting.wordlist_item_4.p2, "",
				 on_shooting_wordlistitem_clicked,
				 on_shooting_wordlistitem_active, 3);

	wordlist_head[0].create (g_pReciteWord->fixed,
				 Skin.shooting.wordlist_head[0].x,
				 Skin.shooting.wordlist_head[0].y,
				 Skin.shooting.wordlist_head[0].p, NULL, NULL,
				 on_shooting_wordlisthead_button_press,
				 GINT_TO_POINTER (0));
	wordlist_head[1].create (g_pReciteWord->fixed,
				 Skin.shooting.wordlist_head[1].x,
				 Skin.shooting.wordlist_head[1].y,
				 Skin.shooting.wordlist_head[1].p, NULL, NULL,
				 on_shooting_wordlisthead_button_press,
				 GINT_TO_POINTER (1));
	wordlist_head[2].create (g_pReciteWord->fixed,
				 Skin.shooting.wordlist_head[2].x,
				 Skin.shooting.wordlist_head[2].y,
				 Skin.shooting.wordlist_head[2].p, NULL, NULL,
				 on_shooting_wordlisthead_button_press,
				 GINT_TO_POINTER (2));
	wordlist_head[3].create (g_pReciteWord->fixed,
				 Skin.shooting.wordlist_head[3].x,
				 Skin.shooting.wordlist_head[3].y,
				 Skin.shooting.wordlist_head[3].p, NULL, NULL,
				 on_shooting_wordlisthead_button_press,
				 GINT_TO_POINTER (3));
	int i;
	for (i = 0; i < 4; i++)
	{
		g_signal_connect (G_OBJECT (wordlist_head[i].event_box),
				  "enter_notify_event",
				  G_CALLBACK
				  (on_shooting_wordlisthead_enter_notify),
				  (void *) i);
		g_signal_connect (G_OBJECT (wordlist_head[i].event_box),
				  "leave_notify_event",
				  G_CALLBACK
				  (on_shooting_wordlisthead_leave_notify),
				  (void *) i);
	}
/*
	int i;
	GtkWidget *pixmap_widget;
	for (i=0;i<4;i++)
	{
		wordlist_head[i] = gtk_event_box_new ();
		gtk_fixed_put (GTK_FIXED (g_pReciteWord->fixed), wordlist_head[i],
		       Skin.shooting.wordlist_head[i].x,
		       Skin.shooting.wordlist_head[i].y);
		pixmap_widget=gtk_image_new_from_pixmap(Skin.shooting.wordlist_head[i].p,(GdkBitmap *) NULL);
		gtk_container_add (GTK_CONTAINER (wordlist_head[i]), pixmap_widget);
		gtk_widget_show(pixmap_widget);
		gtk_widget_show(wordlist_head[i]);

		g_signal_connect (G_OBJECT(wordlist_head[i]), "button_press_event",
			 G_CALLBACK(on_shooting_wordlisthead_button_press), (void *)i);
		g_signal_connect (G_OBJECT(wordlist_head[i]), "enter_notify_event",
			 G_CALLBACK(on_shooting_wordlisthead_enter_notify), (void *)i);
		g_signal_connect (G_OBJECT(wordlist_head[i]), "leave_notify_event",
			 G_CALLBACK(on_shooting_wordlisthead_leave_notify), (void *)i);	
	}
*/
	tank_pixmap =
		gtk_image_new_from_pixmap (Skin.shooting.tank.p1, (GdkBitmap *) NULL);
	gtk_fixed_put (GTK_FIXED (g_pReciteWord->fixed), tank_pixmap,
		       Skin.shooting.tank.x, Skin.shooting.tank.y);
	gtk_widget_show (tank_pixmap);
	shoot_pixmap[0] =
		gtk_image_new_from_pixmap (Skin.shooting.shoot_pixmap[0].p[0],
				(GdkBitmap *) NULL);
	gtk_fixed_put (GTK_FIXED (g_pReciteWord->fixed), shoot_pixmap[0],
		       Skin.shooting.shoot_pixmap[0].x[0],
		       Skin.shooting.shoot_pixmap[0].y[0]);
	shoot_pixmap[1] =
		gtk_image_new_from_pixmap (Skin.shooting.shoot_pixmap[1].p[0],
				(GdkBitmap *) NULL);
	gtk_fixed_put (GTK_FIXED (g_pReciteWord->fixed), shoot_pixmap[1],
		       Skin.shooting.shoot_pixmap[1].x[0],
		       Skin.shooting.shoot_pixmap[1].y[0]);
	shoot_pixmap[2] =
		gtk_image_new_from_pixmap (Skin.shooting.shoot_pixmap[2].p[0],
				(GdkBitmap *) NULL);
	gtk_fixed_put (GTK_FIXED (g_pReciteWord->fixed), shoot_pixmap[2],
		       Skin.shooting.shoot_pixmap[2].x[0],
		       Skin.shooting.shoot_pixmap[2].y[0]);
	shoot_pixmap[3] =
		gtk_image_new_from_pixmap (Skin.shooting.shoot_pixmap[3].p[0],
				(GdkBitmap *) NULL);
	gtk_fixed_put (GTK_FIXED (g_pReciteWord->fixed), shoot_pixmap[3],
		       Skin.shooting.shoot_pixmap[3].x[0],
		       Skin.shooting.shoot_pixmap[3].y[0]);

	right_pixmap[0] =
		gtk_image_new_from_pixmap (Skin.shooting.right_pixmap[0].p1,
				(GdkBitmap *) NULL);
	gtk_fixed_put (GTK_FIXED (g_pReciteWord->fixed), right_pixmap[0],
		       Skin.shooting.right_pixmap[0].x,
		       Skin.shooting.right_pixmap[0].y);
	right_pixmap[1] =
		gtk_image_new_from_pixmap (Skin.shooting.right_pixmap[1].p1,
				(GdkBitmap *) NULL);
	gtk_fixed_put (GTK_FIXED (g_pReciteWord->fixed), right_pixmap[1],
		       Skin.shooting.right_pixmap[1].x,
		       Skin.shooting.right_pixmap[1].y);
	right_pixmap[2] =
		gtk_image_new_from_pixmap (Skin.shooting.right_pixmap[2].p1,
				(GdkBitmap *) NULL);
	gtk_fixed_put (GTK_FIXED (g_pReciteWord->fixed), right_pixmap[2],
		       Skin.shooting.right_pixmap[2].x,
		       Skin.shooting.right_pixmap[2].y);
	right_pixmap[3] =
		gtk_image_new_from_pixmap (Skin.shooting.right_pixmap[3].p1,
				(GdkBitmap *) NULL);
	gtk_fixed_put (GTK_FIXED (g_pReciteWord->fixed), right_pixmap[3],
		       Skin.shooting.right_pixmap[3].x,
		       Skin.shooting.right_pixmap[3].y);

	start_button.create (g_pReciteWord->fixed,
			     Skin.shooting.stop_button.x,
			     Skin.shooting.stop_button.y,
			     Skin.shooting.stop_button.p1,
			     Skin.shooting.stop_button.p2,
			     Skin.shooting.stop_button.p3,
			     Skin.shooting.stop_button.p4,
			     on_shooting_start_clicked);
	pause_button.create (g_pReciteWord->fixed,
			     Skin.shooting.pause_button.x,
			     Skin.shooting.pause_button.y,
			     Skin.shooting.pause_button.p1,
			     Skin.shooting.pause_button.p2,
			     Skin.shooting.pause_button.p3,
			     Skin.shooting.pause_button.p4,
			     on_shooting_pause_clicked);

	on_shooting_start_clicked ();
}

void
CShooting::set_wordlist_label (gint index)
{

	gint tmp[4];
	gint a;

	a = std::rand () % 4;	//the right answer's position
	g_pReciteWord->shooting->tested_wordlist_item[index] = a;

	tmp[a] = index;
	if (a != 0)
	{
		tmp[0] = std::rand () % (g_pReciteWord->now_zu_wordcount);
		while (tmp[0] == tmp[a])
			tmp[0] = std::rand () %
				(g_pReciteWord->now_zu_wordcount);
	}
	
if (a != 1)
	{
		tmp[1] = std::rand () % (g_pReciteWord->now_zu_wordcount);
		while ((tmp[1] == tmp[a]) || (tmp[1] == tmp[0]))
			tmp[1] = std::rand () %
				(g_pReciteWord->now_zu_wordcount);
	}
	
if (a != 2)
	{
		tmp[2] = std::rand () % (g_pReciteWord->now_zu_wordcount);
		while ((tmp[2] == tmp[a]) || (tmp[2] == tmp[0])
		       || (tmp[2] == tmp[1]))
			tmp[2] = std::rand () %
				(g_pReciteWord->now_zu_wordcount);
	}
	
if (a != 3)
	{
		tmp[3] = std::rand () % (g_pReciteWord->now_zu_wordcount);
		while ((tmp[3] == tmp[a]) || (tmp[3] == tmp[0])
		       || (tmp[3] == tmp[1]) || (tmp[3] == tmp[2]))
			tmp[3] = std::rand () %
				(g_pReciteWord->now_zu_wordcount);
	}

	wordlist_item[0].set_label (g_pReciteWord->now_zu_words.m[tmp[0]]);
	wordlist_item[1].set_label (g_pReciteWord->now_zu_words.m[tmp[1]]);
	wordlist_item[2].set_label (g_pReciteWord->now_zu_words.m[tmp[2]]);
	wordlist_item[3].set_label (g_pReciteWord->now_zu_words.m[tmp[3]]);
}

void
CShooting::set_wordview_label (gint index)
{
	gtk_label_set_text (GTK_LABEL (wordview_label_1),
			    g_pReciteWord->now_zu_words.w[index]);
	readword(g_pReciteWord->now_zu_words.w[index]);
}

void
CShooting::close ()
{
	return_button.destroy ();

	if (status == FALSE)
	{
		g_pReciteWord->chooseword->close ();
		return;
	}

	if (shoot_pixmap_timer)
	{
		gtk_timeout_remove (shoot_pixmap_timer);
		shoot_pixmap_timer = 0;
	}

	entocn_robutton.destroy ();
	cntoen_robutton.destroy ();
	read_ckbutton.destroy ();
	wordlist_item[0].destroy ();
	wordlist_item[1].destroy ();
	wordlist_item[2].destroy ();
	wordlist_item[3].destroy ();

	gtk_widget_destroy (wordview_label_1);

	gtk_widget_destroy (tank_pixmap);
	gtk_widget_destroy (shoot_pixmap[0]);
	gtk_widget_destroy (shoot_pixmap[1]);
	gtk_widget_destroy (shoot_pixmap[2]);
	gtk_widget_destroy (shoot_pixmap[3]);
	gtk_widget_destroy (right_pixmap[0]);
	gtk_widget_destroy (right_pixmap[1]);
	gtk_widget_destroy (right_pixmap[2]);
	gtk_widget_destroy (right_pixmap[3]);

	wordlist_head[0].destroy ();
	wordlist_head[1].destroy ();
	wordlist_head[2].destroy ();
	wordlist_head[3].destroy ();

	start_button.destroy ();
	pause_button.destroy ();

	g_free (tested_wordlist_item);
}

void
CShooting::do_up()
{
	if (active_wordlist_item==0)
		return;
	if (!wordlist_head[active_wordlist_item-1].enable)
		return;
	on_shooting_wordlistitem_active (active_wordlist_item-1); //after this fuc,active_wordlist_item=active_wordlist_item-1.
	wordlist_item[active_wordlist_item].active = true;
	wordlist_item[active_wordlist_item].draw ();
}

void
CShooting::do_down()
{
	if (active_wordlist_item==3)
		return;
	if (!wordlist_head[active_wordlist_item+1].enable)
		return;
	on_shooting_wordlistitem_active (active_wordlist_item+1);
	wordlist_item[active_wordlist_item].active = true;
	wordlist_item[active_wordlist_item].draw ();
}

void
CShooting::do_fire()
{
	if (!wordlist_head[active_wordlist_item].enable)
		return;	
	on_shooting_wordlistitem_clicked (active_wordlist_item);
}

CShooting::~CShooting ()
{
}
